//----------------------------------------------------------------------------
// 
//	file: blackboard.h
// 
//	function: blackboard interface to variables 
//		- register all variables relevant to simulation 
//		image.
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Jul 13 06:31:14 CDT 1998
// 
//	$Revision: 1.8 $ $Date: 2001/11/30 20:24:15 $
//	$Log: blackboard.h,v $
//	Revision 1.8  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.7  2000/02/22 03:54:53  yschoe
//	use hash when searching for variables.
//
//	Revision 1.6  1999/02/09 07:37:11  yschoe
//	added list_scalar()
//
//	Revision 1.5  1998/10/16 10:08:04  yschoe
//	added set_parameter(parser*) and force_find(char *)
//
//	Revision 1.4  1998/08/14 06:51:47  yschoe
//	added update() for REAL **
//
//	Revision 1.3  1998/08/11 16:53:58  yschoe
//	minor fixes. Increased buffer size
//
//	Revision 1.2  1998/07/24 09:04:34  yschoe
//	added list()
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------
#ifndef __BLACKBOARD_H
#define __BLACKBOARD_H

#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <search.h>

// #include <endian.h>	this isn't very portable, & it's not used either
#include "real.h"
#include "util.h"
#include "parser.h"

#define	BB_VERSION_STR  "blackboard v0.1"
#define BB_NAME_LEN	100
#define BB_VER_LEN	200
#define BB_COMMENT_LEN	500

// Some useful definitions
enum BB_Type { BB_INT, BB_REAL, BB_RP, BB_RPP, BB_IP, BB_IPP };
enum BB_OP   {BB_READ, BB_WRITE};
const char *const BB_TYPE_NAMES[] = { "BB_INT", "BB_REAL", "BB_RP", 
	"BB_RPP", "BB_IP", "BB_IPP" };

// A union to hold pointers to blackboard entries

typedef union {
	REAL	*r;		// REAL type
	int	*i;		// integer type
	REAL	*rp;		// pointer to REAL array
	int	*ip;		// pointer to integer array
	REAL	**rpp;		// double pointer to REAL matrix
	int	**ipp;		// double pointer to integer matrix
} BB_Data;

// Data size and number of elements
typedef struct {
	char	name[BB_NAME_LEN];	// fixed sized name
	int	size;		// sizeof datatype
	int	cols;		// columns
	int 	rows;		// rows 
	BB_Type type;		// type of data
	BB_Data data;		// the actual data	
} BB_Entry;

//----------------------------------------------------------------------------
//	CLASS: blackboard
//	  - blackboard interface to variables
//	    -- address by name, not by index
//	    -- use a hash function?
//----------------------------------------------------------------------------
class blackboard { 

public:

  char		version[BB_VER_LEN]; // BB version : for later compatibility
  char		comment[BB_COMMENT_LEN]; // BB comment 
  int		count ;	// number of variables in blackboard
			// table[0] .. table[count-1]
  int		max;    // max number of variables

  BB_Entry 	*table;	// table of blackboard entries	
  ENTRY		*hashtab;	// hash table for "name"->"idx"

  int		hash_flag;

	blackboard(char *comment, int max);// constructor: max = max # of vars
	~blackboard();		// destructor

  void	add(char *name, int *var);	// add parameters to blackboard
  void	add(char *name, REAL *var);
  void	add(char *name, int *var, int cols);
  void	add(char *name, REAL *var, int cols);
  void	add(char *name, int **var, int cols, int rows);
  void	add(char *name, REAL **var, int cols, int rows);
  void	update(char *name, REAL **var, int cols, int rows);
  int	set_parameter(char *command);
  int	set_parameter(parser *sim_parser);
  
  // no update functions are necessary -- once the pointers to the 
  // actual variables are set, all updates to the variable
  // will automatically take place  : thus, no need for update()

  int	save(char *filename);	// save blackboard
  int	load(char *filename); // load blackboard

  int  find(char *name);	// find a certain named variable and return idx
  int  find_brute(char *name);	// find a certain named variable and return idx
  int  force_find(char *name);	// find a certain named variable and return idx
  void	list();	// list variable names
  void	list_scalar();	// list variable names

  void build_hash();
  void destroy_hash();

private:

  FILE	*fp;	// for file I/O

  void init(char *name, int size, int cols, int rows, 
		BB_Type type, BB_Entry *bb); 	// init basic values

  //perform actual IO
  //void file_io(BB_OP io, size_t (*io_func)(void *,size_t,size_t,FILE *));
  //int  header_io(BB_OP io, int idx, size_t (*io_func)(void *,size_t,size_t,
  //		FILE *));
  //int  var_io(int idx, size_t (*io_func)(void *,size_t,size_t,FILE *));
  void file_io(BB_OP io);
  int  header_io(BB_OP io, int idx);
  int  var_io(BB_OP io, int idx);
  void add_hash(char *name, int count);

};

#endif 
